import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.event.MouseEvent;

/**
 * Title: HoneycombShape3D
 * Description: This Class combines Geometrie and Appearance for the Honeycomb.
 * It is clickable @see ClickableShape3D and can change Appearance
 * @author Dimitri Missoh & Daniel Schlosser
 * @version 1.0
 */

public class HoneycombShape3D extends Shape3D implements ClickableShape3D
{
           private static final float[] verts = {
    // front face
	 0.5f, -0.5f,  0.5f,
	 0.5f,  0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	-0.5f, -0.5f,  0.5f,
    // back face
	-0.5f, -0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	 0.5f, -0.5f, -0.5f,
    // right face
	 0.5f, -0.5f, -0.5f,
	 0.5f,  0.5f, -0.5f,
	 0.5f,  0.5f,  0.5f,
	 0.5f, -0.5f,  0.5f,
    // left face
	-0.5f, -0.5f,  0.5f,
	-0.5f,  0.5f,  0.5f,
	-0.5f,  0.5f, -0.5f,
	-0.5f, -0.5f, -0.5f,
    // top face
	 0.5f,  0.5f,  0.5f,
	 0.5f,  0.5f, -0.5f,
	-0.5f,  0.5f, -0.5f,
	-0.5f,  0.5f,  0.5f,
    // bottom face
	-0.5f, -0.5f,  0.5f,
	-0.5f, -0.5f, -0.5f,
	 0.5f, -0.5f, -0.5f,
	 0.5f, -0.5f,  0.5f,
    };

    private static final Vector3f[] normals = {
	new Vector3f( 0.0f,  0.0f,  1.0f),	// front face
	new Vector3f( 0.0f,  0.0f, -1.0f),	// back face
	new Vector3f( 1.0f,  0.0f,  0.0f),	// right face
	new Vector3f(-1.0f,  0.0f,  0.0f),	// left face
	new Vector3f( 0.0f,  1.0f,  0.0f),	// top face
	new Vector3f( 0.0f, -1.0f,  0.0f),	// bottom face
    };


       Honeycomb papa;
       Appearance shapeAppearance;
       Material shapeMaterial;
       PolygonAttributes shapePolygonAttributes;
       Color3f ambidif,spec, ambidifsave,specsave;
    public HoneycombShape3D(Honeycomb papa)
    {
       super();
       this.papa=papa;
       ambidifsave=new Color3f();
       specsave=new Color3f();
       QuadArray cube=new QuadArray(24,QuadArray.COORDINATES |QuadArray.NORMALS);
       cube.setCoordinates(0,verts);
       for(int i=0;i<24;i++)
	       cube.setNormal(i,normals[i/4]);
       cube.setCapability(Geometry.ALLOW_INTERSECT);
       this.setGeometry(cube);
       shapeMaterial=new Material();
       shapeMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
       shapeMaterial.setCapability(Material.ALLOW_COMPONENT_READ);
       ambidif=new Color3f(0.0f,0.0f,1.0f);
       shapeMaterial.setAmbientColor(new Color3f(ambidif));
       shapeMaterial.setDiffuseColor(new Color3f(ambidif));
       shapeMaterial.setLightingEnable(true);
       spec=new Color3f(0.1f,0.1f,1.0f);
       shapeMaterial.setSpecularColor(new Color3f(spec));
       shapeMaterial.setShininess(120.0f);
       shapeAppearance=new Appearance();
       shapePolygonAttributes=new PolygonAttributes();
       shapePolygonAttributes.setCapability(PolygonAttributes.ALLOW_MODE_WRITE);
       shapeAppearance.setPolygonAttributes(shapePolygonAttributes);
       shapeAppearance.setCapability(Appearance.ALLOW_MATERIAL_READ);
       shapeAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
       shapeAppearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
       shapeAppearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
       shapeAppearance.setMaterial(shapeMaterial);
       this.setAppearance(shapeAppearance);
    }

    public void changeColor(java.awt.Color col)
    {
	ambidif=new Color3f(col);
	shapeMaterial.setAmbientColor(new Color3f(ambidif));
	shapeMaterial.setDiffuseColor(new Color3f(ambidif));
	spec.set((ambidif.x+0.01f>1.0f)?1.0f:ambidif.x+0.01f,
	        (ambidif.y+0.01f>1.0f)?1.0f:ambidif.y+0.01f,
		(ambidif.z+0.01f>1.0f)?1.0f:ambidif.z+0.01f);
	shapeMaterial.setSpecularColor(new Color3f(spec));
    }
    public void markwithColor(java.awt.Color col)
    {
	Color3f c=new Color3f(col);
        shapeMaterial.setAmbientColor(c);
	shapeMaterial.setDiffuseColor(c);
	shapeMaterial.setSpecularColor((c.x+0.01f>1.0f)?1.0f:c.x+0.01f,
				    (c.y+0.01f>1.0f)?1.0f:c.y+0.01f,
				    (c.z+0.01f>1.0f)?1.0f:c.z+0.01f
					);
    }

    public void demark()
    {
       shapeMaterial.setAmbientColor(ambidif);
       shapeMaterial.setDiffuseColor(ambidif);
       shapeMaterial.setSpecularColor(spec);
    }
    public void toLines(boolean b)
    {
	if(b)
	    shapePolygonAttributes.setPolygonMode(PolygonAttributes.POLYGON_LINE);
    	else
	    shapePolygonAttributes.setPolygonMode(PolygonAttributes.POLYGON_FILL);
    }
    public void ClickActionAppeared(MouseEvent event)
    {
	papa.ClickActionAppeared(event);
    }
}
