import java.util.LinkedList;

/** Der Baum, der vom PatternGenerator aufgebaut wird, besteht aus Knoten.<br>
 * Ein Pattern wird in den Baum eingefgt, indem fr einen Index in das Array, das das Pattern reprsentiert, geschaut wird,
 * ob dort ein Stein ist, oder nicht (auch ob ein Wildcard dort steht). Je nachdem wird in einen anderen Teilbaum weitergegangen
 * und der Index um eins erhht. Es handelt sich um eine triren Baum, die Nachfolger werden unterschieden in "dort ist ein Stein",
 * "dort ist kein Stein" und "dort ist ein Joker"). Wobei "dort" durch den Index angegeben wird.
 *  Der Index ist also auch gleichbedeutend mit der Tiefe des Knotens.
 *
 * Stand: 09.07.2001
 */
public class Knoten{
    protected int wert;

    protected Knoten steinNachfolger;
    protected Knoten leerNachfolger;
    protected Knoten jokerNachfolger;

    /** Der Konstruktor fr einen leeren Baum.
     */
    public Knoten(){
        wert = 0;
    }

    /** Dieser Konstruktor wird aufgerufen, wenn an dieser Stelle im Baum noch kein Knoten war.
     * Es wird also die einzufgende Regel und die Tiefe dieses Knotens im Baum bentigt und also auch bergeben.
     *
     * @param newRule einzufgende Regel
     * @param newWert Tiefe dieses Knotens im Baum
     */
    protected Knoten( Rule newRule, int newWert ){
        wert = newWert;
        try{
            addRule( newRule );
        }catch( PatternNotAddableInTreeException e ){
            System.out.println( "Kann Pattern nicht in Baum einfgen." );
        }
    }
  


    /** Diese Methode liefert fr ein Sichtfeld der Ameise alle mglichen Actions als LinkedList indem einfach im Baum abgestiegen wird.
     *
     * Gendert: 25.06.2001 umgestellt auf Joker-betrieb. Energie!
     *           04.07.2001 grosses try-catch um praktisch alle Anweisungen in der Methoded (NullPointer)
     *
     * @param view Blickfeld
     * @param positionAgent Position des Agenten in der Nest
     * @return LinkedList aller mglichen Actions
     */
    public LinkedList getActionFromBlickfeld( Blickfeld view, Koord3D positionAgent )
        throws PatternNotFoundInTreeException{
	LinkedList tempList1 = null;
	LinkedList tempList2 = null;
		
        try{     // 04.07.2001 (Steffen)

            if( jokerNachfolger != null ){		   

                try{
                    tempList1 = jokerNachfolger.getActionFromBlickfeld( view, positionAgent );			
                }catch( PatternNotFoundInTreeException e ){		
                }catch(NullPointerException e ){
                }      


                if( view.getStateAt( wert, positionAgent ) == Nest.STEIN ){		    		 		    
                    try{
                        tempList2 = steinNachfolger.getActionFromBlickfeld( view, positionAgent );
                    }catch(PatternNotFoundInTreeException e ){
                    }catch(NullPointerException e ){
                    }

                    if( tempList1 != null && tempList2 != null ){
                        tempList1.addAll( tempList2 );
                        return tempList1;
                    }else if( tempList1 != null ){
                        return tempList1;
                    }else if( tempList2 != null ){
                        return tempList2;
                    }else{		    
                        throw new PatternNotFoundInTreeException();
                    }		  
                }else{		    		   		    
                    try{
                        tempList2 = leerNachfolger.getActionFromBlickfeld( view, positionAgent );
                    }catch(PatternNotFoundInTreeException e ){
                    }catch(NullPointerException e ){
                    }

                    if( tempList1 != null && tempList2 != null ){
                        tempList1.addAll( tempList2 );
                        return tempList1;
                    }else if( tempList1 != null ){
                        return tempList1;
                    }else if( tempList2 != null ){
                        return tempList2;
                    }else{		   
                        throw new PatternNotFoundInTreeException();
                    }		  		 
                }
            }else{
                if( view.getStateAt( wert, positionAgent ) == Nest.STEIN ){
                    return steinNachfolger.getActionFromBlickfeld( view, positionAgent );
                }else{
                    return leerNachfolger.getActionFromBlickfeld( view, positionAgent );
                }
            }
        }catch( NullPointerException e ){
            throw new PatternNotFoundInTreeException();
        }
 
    }

    /** Mit dieser Methode wird eine Regel in den Baum eingefgt, wenn ein Knoten bereits existiert.
     *
     * Gendert: ?
     *
     * @param newRule einzufgende Regel   
     */
    public void addRule( Rule newRule )
        throws PatternNotAddableInTreeException{

        if( wert == ( newRule.size()-1 ) ){
            switch( newRule.getStateAt( wert ) ){
            case Nest.STEIN:
                if( steinNachfolger == null ){
                    steinNachfolger = new Blatt( newRule );
                }else{
                    ((Blatt)steinNachfolger).addAction( newRule.getAction() );
                }
                break;

            case Nest.JOKER:
		
                if( jokerNachfolger == null ){
                    jokerNachfolger = new Blatt( newRule );
                }else{
                    ((Blatt)jokerNachfolger).addAction( newRule.getAction() );
                }
               
                break;

            case Nest.LEER:
                if( leerNachfolger == null ){
                    leerNachfolger = new Blatt( newRule );
                }else{
                    ((Blatt)leerNachfolger).addAction( newRule.getAction() );
                }
                break;

            default:		
                throw new PatternNotAddableInTreeException();
            }
        }else{
            switch ( newRule.getStateAt( wert ) ){
            case Nest.STEIN:
                if( steinNachfolger == null ){
                    steinNachfolger = new Knoten( newRule, wert + 1 );
                }else{
                    steinNachfolger.addRule( newRule );
                }
                break;

            case Nest.JOKER:
		
		if( jokerNachfolger == null ){
                    jokerNachfolger = new Knoten( newRule, wert + 1 );
                }else{
                    jokerNachfolger.addRule( newRule );
                }               
                break;

            case Nest.LEER:
                if( leerNachfolger == null ){
                    leerNachfolger = new Knoten( newRule, wert + 1 );
                }else{
                    leerNachfolger.addRule( newRule );
                }
                break;

            default:
                throw new PatternNotAddableInTreeException();
            }
        }
    }
}

